# -*- coding: utf-8 -*-
from hs.api.futures_constant import FuturesOrderOptions
from hs.api.hq_constant import MktTmType
from hs.api.trade_constant import ClientType, EntrustType, SessionType
from hs.common.common_utils import get_info_logger
from hs.common.request_constant import CommonUrl, TradeUrl, HqUrl
from hs.common.socket_client import SocketClient

logging_name: str = __name__


class OpenAPI(object):
    """
    开箱即用API
    """

    def __init__(self, ip: str, http_port: str, tcp_port: str, logging_filename: str = None):
        logger = get_info_logger(logging_name, filename=logging_filename)
        self._logging = logger
        self._socket_client = SocketClient(ip=ip, http_port=http_port, tcp_port=tcp_port, logger=logger)

    def is_alive(self):
        """
        检查StockClient是否正常连接状态
        """
        return self._socket_client.is_alive()

    def start(self):
        """启动业务API上下文环境，重启StockClient"""
        self._socket_client.restart()

    def add_notify_callback(self, callback):
        """增加消息推送回调函数"""
        self._socket_client.handle_notify_for_ever(callback)

    def stop(self):
        """退出业务API上下文环境，停止StockClient"""
        self._socket_client.stop()

    ##########################################      Common API      ##################################################

    def query_rate_list(self, rate_type):
        """
        查询汇率列表
        :param rate_type: 汇率类型  hs.api.trade_constant.RateType
        """
        return self._socket_client.post_request(request_url=CommonUrl.HS_QUERY_RATE_LIST,
                                                request_data={'rateType': rate_type})

    ##########################################      Trading API      ##################################################

    def trade_login(self, password: str):
        """
        交易登录
        :param password: 交易登录密码
        """
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_LOGIN_REQUEST_MSG_TYPE,
                                                request_data={'password': password})

    def trade_logout(self):
        """
        交易登出
        """
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_LOGOUT_REQUEST_MSG_TYPE,
                                                request_data={})

    def query_margin_fund_info(self, exchange_type: str):
        """
        查询客户资金信息
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        """
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_MARGIN_FUND_INFO_REQUEST_MSG_TYPE,
                                                request_data={'exchangeType': exchange_type})

    def query_holds_list(self, exchange_type: str):
        """
        查询持仓列表
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        """
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_HOLDS_LIST_REQUEST_MSG_TYPE,
                                                request_data={'exchangeType': exchange_type})

    def query_buy_amount(self, exchange_type: str, stock_code: str, entrust_price: str):
        """
        查询最大可买
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        :param entrust_price: 委托价格
        """
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code,
            'entrustPrice': entrust_price,
            'clientType': ClientType.INTERNET
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_BUY_AMOUNT_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_sell_amount(self, exchange_type: str, stock_code: str):
        """
        查询最大可卖
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        """
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code,
            'clientType': ClientType.INTERNET
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_SELL_AMOUNT_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_real_fund_jour_list(self, exchange_type: str, query_count: int = 20, query_param_str: str = "0"):
        """
        查询当日资金流水
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param query_count: 每页返回数量 默认20（限制小于100）
        :param query_param_str: 游标 初始值为0 开始
        """
        request_data = {
            'exchangeType': exchange_type,
            'queryCount': query_count,
            'queryParamStr': query_param_str
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_REAL_FUND_JOUR_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_history_fund_jour_list(self, exchange_type: str,
                                     start_date: str,
                                     end_date: str,
                                     query_count: int = 20,
                                     query_param_str: str = "0"):
        """
        查询历史资金流水
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param start_date: 起始日期 格式为：yyyyMMdd
        :param end_date: 结束日期 格式为：yyyyMMdd
        :param query_count: 每页返回数量 默认20（限制小于100）
        :param query_param_str: 游标 初始值为0 开始
        """
        request_data = {
            'exchangeType': exchange_type,
            'startDate': start_date,
            'endDate': end_date,
            'queryCount': query_count,
            'queryParamStr': query_param_str
        }
        return self._socket_client.post_request(
            request_url=TradeUrl.TRADE_QUERY_HISTORY_FUND_JOUR_LIST_REQUEST_MSG_TYPE,
            request_data=request_data)

    def entrust(self, exchange_type: str,
                stock_code: str,
                entrust_bs: str,
                entrust_type: str,
                entrust_amount: str,
                entrust_price: str,
                session_type: str,
                exchange: str,
                iceberg_display_size: str,
                cond_track_type: str = None,
                cond_value: str = None,
                valid_days: str = None):
        """
        委托下单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        :param entrust_bs: 买卖方向   hs.api.trade_constant.EntrustBS
        :param entrust_type: 委托类型 可能值：港股：'0'竞价限价、'1'竞价、'2'增强限价盘、'3'限价盘、'4'特别限价盘、'6'暗盘。
                                          美股：'3'限价盘、'5'市价盘。
                                          A股：'3'限价盘。
                                          条件单：'31'-止盈限价单、'32'-止盈市价单(美股)、'33'-止损限价单、'34'-止损市价单(美股)、'35'-追踪止损限价单、'36'-追踪止损市价单(美股)
                                          hs.api.trade_constant.EntrustType
        :param entrust_amount: 委托数量
        :param entrust_price: 委托价格
        :param session_type: 盘前盘后交易  
                                0否 1是 3只支持盘中 6美股只支持盘前盘后 7美股支持盘中及盘前盘后 8美股只支持盘前 9美股只支持盘后
                                hs.api.trade_constant.SessionType
        :param exchange: 交易所  hs.api.trade_constant.EntrustEX
        :param iceberg_display_size: 冰山单披露数量 
        :param cond_track_type: 跟踪类型  1百分比、2价差、3价格  hs.api.trade_constant.CondTrackType
        :param cond_value: 触发条件值 价格/价差/百分比值
        :param valid_days: 有效天数 
        """
        if len(session_type) == 0 and entrust_type in (EntrustType.COND_STOP_PROFIT_POINT,
                                                       EntrustType.COND_STOP_PROFIT_MARKET,
                                                       EntrustType.COND_STOP_LOSS_POINT,
                                                       EntrustType.COND_STOP_LOSS_MARKET,
                                                       EntrustType.COND_TRACK_STOP_LOSS_POINT,
                                                       EntrustType.COND_TRACK_STOP_LOSS_MARKET):
            session_type = SessionType.COND_ORDER_DEFAULT
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code,
            'entrustBs': entrust_bs,
            'entrustType': entrust_type,
            'entrustPrice': entrust_price,
            'entrustAmount': entrust_amount,
            'clientType': ClientType.INTERNET
        }
        if session_type is not None:
            request_data['sessionType'] = session_type
        if exchange is not None:
            request_data['exchange'] = exchange
        if iceberg_display_size is not None:
            request_data['iceBergDisplaySize'] = iceberg_display_size
        if cond_track_type is not None:
            request_data['condTrackType'] = cond_track_type
        if cond_value is not None:
            request_data['condValue'] = cond_value
        if valid_days is not None:
            request_data['validDays'] = valid_days
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def cancel_entrust(self, entrust_id: str,
                       exchange_type: str,
                       stock_code: str,
                       entrust_amount: str = None,
                       entrust_price: str = None,
                       entrust_type: str = None, ):
        """
        撤单
        :param entrust_id: 订单ID
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        :param entrust_amount: 委托数量、必须传原委托的委托数量
        :param entrust_price: 委托价格
        :param entrust_type: 委托类型  hs.api.trade_constant.EntrustType
        """
        request_data = {
            'entrustId': entrust_id,
            'exchangeType': exchange_type,
            'stockCode': stock_code
        }
        if entrust_amount is not None:
            request_data['entrustAmount'] = entrust_amount
        if entrust_price is not None:
            request_data['entrustPrice'] = entrust_price
        if entrust_type is not None:
            request_data['entrustType'] = entrust_type
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_CANCEL_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def batch_cancel_entrust(self, exchange_type: str, entrust_ids: list = None):
        """
        批量撤单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param entrust_ids: 订单ID列表
        """
        request_data = {
            'exchangeType': exchange_type,
            'entrustId': entrust_ids
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_BATCH_CANCEL_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def change_entrust(self, exchange_type: str,
                       entrust_id: str,
                       stock_code: str,
                       entrust_amount: str,
                       entrust_price: str,
                       entrust_type: str = None,
                       session_type: str = None,
                       cond_track_type: str = None,
                       cond_value: str = None,
                       valid_days: str = None):
        """
        改单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param entrust_id: 要改单的原委托编号
        :param stock_code: 证券代码
        :param entrust_amount: 修改委托数量
        :param entrust_price: 修改委托价格
        :param entrust_type: 委托类型  hs.api.trade_constant.EntrustType
        :param session_type: 盘前盘后交易 
                                3只支持盘中 6美股只支持盘前盘后 7美股支持盘中及盘前盘后
                                hs.api.trade_constant.SessionType
        :param cond_track_type: 跟踪类型 1百分比、2价差、3价格  hs.api.trade_constant.CondTrackType
        :param cond_value: 触发条件值 价格/价差/百分比值
        :param valid_days: 有效天数
        """
        request_data = {
            'exchangeType': exchange_type,
            'entrustId': entrust_id,
            'stockCode': stock_code,
            'entrustAmount': entrust_amount,
            'entrustPrice': entrust_price,
        }
        if entrust_type is not None:
            request_data['entrustType'] = entrust_type
        if session_type is not None:
            request_data['sessionType'] = session_type
        if cond_track_type is not None:
            request_data['condTrackType'] = cond_track_type
        if cond_value is not None:
            request_data['condValue'] = cond_value
        if valid_days is not None:
            request_data['validDays'] = valid_days
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_CHANGE_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_max_available_asset(self, exchange_type: str, stock_code: str, entrust_price: str, entrust_type: str):
        """
        查询最大可用资产
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        :param entrust_price: 委托价格
        :param entrust_type: 委托类型  hs.api.trade_constant.EntrustType
        """
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code,
            'entrustPrice': entrust_price,
            'entrustType': entrust_type
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_MAX_AVAILABLE_ASSET_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_real_entrust_list(self, exchange_type: str, query_count: int = 20, query_param_str: str = "0",
                                entrust_ids: list = []):
        """
        查询当日委托
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param query_count: 每页返回数量 默认20（限制小于100）
        :param query_param_str: 游标 初始值为0 开始
        :param entrust_ids: 订单ID，可选
        """
        request_data = {
            'exchangeType': exchange_type,
            'queryCount': query_count,
            'queryParamStr': query_param_str,
            'entrustId': entrust_ids
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_REAL_ENTRUST_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_history_entrust_list(self, exchange_type: str,
                                   start_date: str,
                                   end_date: str,
                                   query_count: int = 20,
                                   query_param_str: str = "0"):
        """
        查询历史委托
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param start_date: 起始日期 格式为：yyyyMMdd
        :param end_date: 结束日期 格式为：yyyyMMdd
        :param query_count: 每页返回数量 默认20（限制小于100）
        :param query_param_str: 游标 初始值为0 开始
        """
        request_data = {
            'exchangeType': exchange_type,
            'startDate': start_date,
            'endDate': end_date,
            'queryCount': query_count,
            'queryParamStr': query_param_str
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_HISTORY_ENTRUST_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_real_deliver_list(self, exchange_type: str, query_count: int = 20, query_param_str: str = "0"):
        """
        查询当日成交
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param query_count: 每页返回数量 默认20（限制小于100）
        :param query_param_str: 游标 初始值为0 开始
        """
        request_data = {
            'exchangeType': exchange_type,
            'queryCount': query_count,
            'queryParamStr': query_param_str
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_REAL_DELIVER_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_history_deliver_list(self, exchange_type: str,
                                   start_date: str,
                                   end_date: str,
                                   query_count: int = 20,
                                   query_param_str: str = "0"):
        """
        查询历史成交
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param start_date: 起始日期 格式为：yyyyMMdd
        :param end_date: 结束日期 格式为：yyyyMMdd
        :param query_count: 每页返回数量 默认20（限制小于100）
        :param query_param_str: 游标 初始值为0 开始
        """
        request_data = {
            'exchangeType': exchange_type,
            'startDate': start_date,
            'endDate': end_date,
            'queryCount': query_count,
            'queryParamStr': query_param_str
        }
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_HISTORY_DELIVER_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_real_cond_order_list(self, exchange_type: str, stock_code: str, page_no: int = 1, page_size: int = 20):
        """
        查询当日条件单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 股票代码
        :param page_no: 页码，从1开始
        :param page_size: 每页数量，限制小于100
        """
        request_data = {
            'exchangeType': exchange_type,
            'pageNo': page_no,
            'pageSize': page_size
        }
        if stock_code is not None:
            request_data['stockCode'] = stock_code
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_QUERY_REAL_COND_ORDER_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_history_cond_order_list(self, exchange_type: str,
                                      stock_code: str,
                                      start_time: str,
                                      end_time: str,
                                      page_no: int = 1,
                                      page_size: int = 20):
        """
        查询历史条件单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 股票代码 
        :param start_time: 查询开始时间 
        :param end_time: 查询结束时间 
        :param page_no: 页码，从1开始
        :param page_size: 每页数量，限制小于100
        """
        request_data = {
            'exchangeType': exchange_type,
            'startTime': start_time,
            'endTime': end_time,
            'pageNo': page_no,
            'pageSize': page_size
        }
        if stock_code is not None:
            request_data['stockCode'] = stock_code
        return self._socket_client.post_request(
            request_url=TradeUrl.TRADE_QUERY_HISTORY_COND_ORDER_LIST_REQUEST_MSG_TYPE,
            request_data=request_data)

    def query_before_and_after_support(self, exchange_type: str, stock_code: str):
        """
        查询是否支持盘前盘后交易
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        """
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code
        }
        return self._socket_client.post_request(
            request_url=TradeUrl.TRADE_QUERY_BEFORE_AND_AFTER_SUPPORT_REQUEST_MSG_TYPE,
            request_data=request_data)

    def query_stock_short_info(self, exchange_type: str, stock_code: str):
        """
        查询股票沽空信息
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        """
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code
        }
        return self._socket_client.post_request(
            request_url=TradeUrl.TRADE_QUERY_STOCK_SHORT_INFO_REQUEST_MSG_TYPE,
            request_data=request_data)

    def query_margin_full_info(self, data_type: str, stock_code: str):
        """
        查询股票融资融券数据
        :param data_type: 股票类型  hs.api.hq_constant.DataType
        :param stock_code: 证券代码
        """
        request_data = {
            'dataType': data_type,
            'stockCode': stock_code
        }
        return self._socket_client.post_request(
            request_url=TradeUrl.TRADE_QUERY_MARGIN_FULL_INFO_REQUEST_MSG_TYPE,
            request_data=request_data)

    def trade_subscribe(self):
        """
        订阅交易推送
        """
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_SUBSCRIBE_REQUEST_MSG_TYPE,
                                                request_data={})

    def trade_unsubscribe(self):
        """
        取消订阅交易推送
        """
        return self._socket_client.post_request(request_url=TradeUrl.TRADE_UNSUBSCRIBE_REQUEST_MSG_TYPE,
                                                request_data={})

    def algo_entrust(self, stock_code: str,
                     exchange_type: str,
                     entrust_type: str,
                     entrust_price: str,
                     entrust_amount: str,
                     entrust_bs: str,
                     target_strategy: str,
                     session_type: str,
                     max_volume: str,
                     sensitivity: str,
                     orig_start_time: str = None,
                     orig_end_time: str = None,
                     min_amount: str = None,
                     show_qty: str = None,
                     qty_percent: str = None,
                     interval: int = 64):
        """
        策略单-下单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param stock_code: 证券代码
        :param entrust_type: 委托类型 [1:限价单 2:市价单]  hs.api.trade_constant.AlgoEntrustType
        :param entrust_price: 委托价格
        :param entrust_amount: 委托数量
        :param entrust_bs: 交易方向 [1:买入 2:卖出] hs.api.trade_constant.EntrustBS
        :param target_strategy: 策略类型 
                                    [1:VWAP 1001:TWAP 1002:ICE_BERG 1003:TPOV 1005:INLINE]  
                                    hs.api.trade_constant.AlgoStrategyType
        :param session_type: 盘前盘后交易 [0:否 1:是]  hs.api.trade_constant.SessionType

        :param max_volume: 每笔子单最大的委托数量 
        :param sensitivity: 订单的执行效率[1:中性(neutral) 2:主动(aggressive) 3:被动(passive)]
        :param orig_start_time: 开始时间，格式为HHmmSS
        :param orig_end_time: 结束时间，格式为HHmmSS
        :param min_amount: 每笔子单的最小交易金额
        :param show_qty: IceBerg订单使用， 每次的报单数量
        :param qty_percent: TPOV策略使用订单盘口占比, Inline使用累计成交占比（可设置1-99）
        :param interval: 订单报单间隔，Inline策略使用（默认60）
        """
        strategy_param = {
            'maxVolume': max_volume,
            'sensitivity': sensitivity,
            'origStartTime': orig_start_time,
            'origEndTime': orig_end_time,
            'minAmount': min_amount,
            'showQty': show_qty,
            'qtyPercent': qty_percent,
            'interval': interval
        }
        request_data = {
            'exchangeType': exchange_type,
            'stockCode': stock_code,
            'entrustType': entrust_type,
            'entrustBs': entrust_bs,
            'entrustAmount': entrust_amount,
            'entrustPrice': entrust_price,
            'targetStrategy': target_strategy,
            'sessionType': session_type,
            'strategyParam': strategy_param
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_ADD_ORDER_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def algo_cancel_order(self, exchange_type: str, order_id: str):
        """
        策略单-撤销母单
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param order_id: 母订单号
        """
        request_data = {
            'exchangeType': exchange_type,
            'orderId': order_id
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_CANCEL_ORDER_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def algo_cancel_sub_order(self, exchange_type: str, order_id: str, sub_order_id: str):
        """
        策略单-撤销子单
        :param exchange_type: 市场
        :param order_id: 母订单号
        :param sub_order_id: 子订单号
        """
        request_data = {
            'exchangeType': exchange_type,
            'orderId': order_id,
            'entrustId': sub_order_id
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_CANCEL_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def algo_modify_order(self, order_id: str,
                          exchange_type: str,
                          stock_code: str,
                          entrust_price: str,
                          entrust_amount: str,
                          orig_start_time: str = None,
                          orig_end_time: str = None,
                          max_volume: str = None,
                          min_amount: str = None,
                          sensitivity: str = None,
                          show_qty: str = None,
                          qty_percent: str = None):
        """
        策略单-改单
        :param order_id: 母订单号
        :param stock_code: 证券代码
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param entrust_price: 委托价格
        :param entrust_amount: 委托数量

        :param max_volume: 每笔子单最大的委托数量
        :param sensitivity: 订单的执行效率
        :param orig_start_time: 开始时间
        :param orig_end_time: 结束时间
        :param min_amount: 每笔子单的最小交易金额
        :param show_qty: <IceBerg策略使用> 每次的报单数量
        :param qty_percent: TPOV策略使用订单盘口占比, Inline使用累计成交占比（可设置1-99）
        """
        strategy_param = {
            'maxVolume': max_volume,
            'sensitivity': sensitivity,
            'origStartTime': orig_start_time,
            'origEndTime': orig_end_time,
            'minAmount': min_amount,
            'showQty': show_qty,
            'qtyPercent': qty_percent
        }
        request_data = {
            'orderId': order_id,
            'exchangeType': exchange_type,
            'stockCode': stock_code,
            'entrustAmount': entrust_amount,
            'entrustPrice': entrust_price,
            'strategyParam': strategy_param
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_CHANGE_ORDER_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def algo_order_action(self, order_id: str,
                          exchange_type: str,
                          action: str,
                          target_strategy: str = None):
        """
        策略单-操作
        :param order_id: 母订单号
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        :param action: 操作 [1:START 2:STOP 3:SUSPEND 4:RESUME]  hs.api.trade_constant.AlgoActionType
        :param target_strategy: 策略类型 
                                    [1:VWAP 1001:TWAP 1002:ICE_BERG 1003:TPOV 1005:INLINE]
                                    hs.api.trade_constant.AlgoStrategyType
        """
        request_data = {
            'orderId': order_id,
            'exchangeType': exchange_type,
            'action': action,
            'targetStrategy': target_strategy
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_ACTION_ORDER_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def algo_query_order_list(self, start_date: str,
                              end_date: str,
                              page_no: int = 1,
                              page_size: int = 30,
                              exchange_type: str = None,
                              stock_code: str = None):
        """
        策略单-查询母单列表
        :param start_date: 开始日期 yyyyMMdd
        :param end_date: 结束日期 yyyyMMdd
        :param page_no: 当前页号
        :param page_size: 每页条数
        :param exchange_type: 市场
        :param stock_code: 股票代码
        """
        request_data = {
            'pageNo': page_no,
            'pageSize': page_size,
            'startDate': start_date,
            'endDate': end_date,
            'exchange_type': exchange_type,
            'stock_code': stock_code
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_QUERY_ORDER_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def algo_query_sub_order_id_list(self, order_id: str,
                                     trade_date: str,
                                     exchange_type: str):
        """
        策略单-查询子单
        :param order_id: 母单ID
        :param trade_date: 交易日 yyyyMMdd
        :param exchange_type: 交易类型  hs.api.trade_constant.ExchangeType
        """
        request_data = {
            'orderId': order_id,
            'tradeDate': trade_date,
            'exchangeType': exchange_type
        }
        return self._socket_client.post_request(request_url=TradeUrl.ALGO_QUERY_ENTRUST_ID_LIST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    ########################################## Futures API ##################################################

    def futures_query_fund_info(self):
        """
        期货-查询客户资金信息
        """
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_QUERY_FUND_INFO_REQUEST_MSG_TYPE,
                                                request_data={})

    def futures_query_holds_list(self):
        """
        期货-查询持仓列表
        """
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_QUERY_HOLDS_LIST_REQUEST_MSG_TYPE,
                                                request_data={})

    def futures_query_max_buy_sell_amount(self, stock_code: str):
        """
        期货-查询最大可买可卖
        :param stock_code 期货合约代码
        """
        request_data = {
            'stockCode': stock_code
        }
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_QUERY_MAX_BUY_SELL_AMOUNT_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def futures_query_product_info(self, stock_code_list: list):
        """
        期货-查询合约产品信息
        :param stock_code_list 期货合约代码
        """
        request_data = {
            'stockCode': stock_code_list
        }
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_QUERY_PRODUCT_INFO_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def futures_entrust(self, stock_code: str,
                        entrust_type: str,
                        entrust_price: str,
                        entrust_amount: str,
                        entrust_bs: str,
                        valid_time_type: str,
                        valid_time: str,
                        order_options: str = FuturesOrderOptions.DEFAULT):
        """
        期货-下单
        :param stock_code 期货合约代码
        :param entrust_type 委托类型  hs.api.futures_constant.FuturesEntrustType
        :param entrust_price 委托价格
        :param entrust_amount 委托数量
        :param entrust_bs  买卖方向   hs.api.futures_constant.FuturesEntrustBs
        :param valid_time_type 生效类型  hs.api.futures_constant.FuturesValidTimeType
        :param valid_time  当 valid_time_type==4 必填yyyyMMdd
        :param order_options  订单选项  hs.api.futures_constant.FuturesOrderOptions
        """
        request_data = {
            'stockCode': stock_code,
            'entrustType': entrust_type,
            'entrustPrice': entrust_price,
            'entrustAmount': entrust_amount,
            'entrustBs': entrust_bs,
            'validTimeType': valid_time_type,
            'orderOptions': order_options
        }
        if valid_time is not None:
            request_data['validTime'] = valid_time
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def futures_cancel_entrust(self, entrust_id: str, stock_code: str):
        """
        期货-撤单
        :param entrust_id 期货订单ID
        :param stock_code 期货合约代码
        """
        request_data = {
            'entrustId': entrust_id,
            'stockCode': stock_code
        }
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_CANCEL_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def futures_change_entrust(self, entrust_id: str,
                               stock_code: str,
                               entrust_price: str,
                               entrust_amount: str,
                               entrust_bs: str,
                               valid_time_type: str,
                               valid_time: str,
                               order_options: str = FuturesOrderOptions.DEFAULT):
        """
        期货-改单
        :param entrust_id 期货订单ID
        :param stock_code 期货合约代码
        :param entrust_price 委托价格
        :param entrust_amount 委托数量
        :param entrust_bs  买卖方向  hs.api.futures_constant.FuturesEntrustBs
        :param valid_time_type 生效类型 hs.api.futures_constant.FuturesValidTimeType
        :param valid_time 当 valid_time_type==4 必填yyyyMMdd
        :param order_options 订单选项  hs.api.futures_constant.FuturesOrderOptions
        """
        request_data = {
            'entrustId': entrust_id,
            'stockCode': stock_code,
            'entrustPrice': entrust_price,
            'entrustAmount': entrust_amount,
            'entrustBs': entrust_bs,
            'validTimeType': valid_time_type,
            'orderOptions': order_options
        }
        if valid_time is not None:
            request_data['validTime'] = valid_time
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_MODIFY_ENTRUST_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def futures_query_real_entrust_list(self):
        """
        期货-查询当日委托
        """
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_QUERY_REAL_ENTRUST_LIST_REQUEST_MSG_TYPE,
                                                request_data={})

    def futures_query_history_entrust_list(self, page_no: int, page_size: int, start_date: str, end_date: str):
        """
        期货-查询历史委托
        :param page_no 页码 默认1
        :param page_size 每页返回数量 默认20
        :param start_date 起始日期 格式为：yyyyMMdd
        :param end_date 结束日期 格式为：yyyyMMdd
        """
        request_data = {
            'pageNo': page_no,
            'pageSize': page_size,
            'startDate': start_date,
            'endDate': end_date
        }
        return self._socket_client.post_request(
            request_url=TradeUrl.FUTURES_QUERY_HISTORY_ENTRUST_LIST_REQUEST_MSG_TYPE,
            request_data=request_data)

    def futures_query_real_deliver_list(self):
        """
        期货-查询当日成交
        """
        return self._socket_client.post_request(request_url=TradeUrl.FUTURES_QUERY_REAL_DELIVER_LIST_REQUEST_MSG_TYPE,
                                                request_data={})

    def futures_query_history_deliver_list(self, page_no: int, page_size: int, start_date: str, end_date: str):
        """
        期货-查询历史成交
        :param page_no 页码 默认1
        :param page_size 每页返回数量 默认20
        :param start_date 起始日期 格式为：yyyyMMdd
        :param end_date 结束日期 格式为：yyyyMMdd
        """
        request_data = {
            'pageNo': page_no,
            'pageSize': page_size,
            'startDate': start_date,
            'endDate': end_date
        }
        return self._socket_client.post_request(
            request_url=TradeUrl.FUTURES_QUERY_HISTORY_DELIVER_LIST_REQUEST_MSG_TYPE,
            request_data=request_data)

    ##########################################      Hq API    ##################################################

    def query_hq_basic_qot(self, security_list: list, needDelayFlag="", mkt_tm_type: int = MktTmType.MID_SESSION):
        """
        查询基础报价
        :param security_list list[SecurityParam] 证券信息
        :param needDelayFlag 是否绕开原有的权限判断体系，目前只支持港股市场与美股市场的延时权限下的延时数据 0：否，1：是
        :param mkt_tm_type 交易时段。美股：盘前：-1 盘中：1 盘后：-2；港股可以设置3个中的任意值
        """
        request_data = {
            'security': security_list,
            'needDelayFlag':needDelayFlag,
            'mktTmType':mkt_tm_type
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_BASIC_QOT_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_broker(self, security_param: dict):
        """
        查询买卖经纪摆盘
        :param security_param 证券信息
        """
        request_data = {
            'security': security_param
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_BROKER_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_order_book(self, security_param: dict,
                            mkt_tm_type: int = MktTmType.MID_SESSION,
                            depth_book_type: int = None):
        """
        查询买卖档
        :param security_param 证券信息
        :param mkt_tm_type 交易时段。美股：盘前：-1 盘中：1 盘后：-2；港股可以设置3个中的任意值
        :param depth_book_type 深度摆盘
        """
        request_data = {
            'security': security_param,
            'mktTmType': mkt_tm_type
        }
        if depth_book_type is not None:
            request_data['depthBookType'] = depth_book_type
        return self._socket_client.post_request(request_url=HqUrl.HQ_ORDER_BOOK_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_ticker(self, security_param: dict, limit: int, mkt_tm_type: int = MktTmType.MID_SESSION):
        """
        查询逐笔
        :param security_param 证券信息
        :param limit 返回的逐笔个数，实际返回数量不一定会返回这么多,最多返回1000个
        :param mkt_tm_type 交易时段。美股：盘前：-1 盘中：1 盘后：-2；港股可以设置3个中的任意值
        """
        request_data = {
            'security': security_param,
            'limit': limit,
            'mktTmType':mkt_tm_type
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_TICKER_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_kline(self, security_param: dict,
                       start_date: str,
                       direction: int,
                       ex_right_flag: int,
                       cyc_type: int,
                       limit: int):
        """
        查询K线
        :param security_param 证券信息
        :param start_date 起始日期 格式为：yyyyMMdd
        :param direction 查询方向  hs.api.constant.Direction
        :param ex_right_flag 复权类型  hs.api.constant.ExRightFlag
        :param cyc_type K线类型  hs.api.constant.CycType
        :param limit 查询k线数量限制
        """
        request_data = {
            'security': security_param,
            'startDate': start_date,
            'direction': direction,
            'exRightFlag': ex_right_flag,
            'cycType': cyc_type,
            'limit': limit
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_KL_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_time_share(self, security_param: dict, mkt_tm_type: int = MktTmType.MID_SESSION):
        """
        查询分时
        :param security_param 证券信息
        :param mkt_tm_type 交易时段。美股：盘前：-1 盘中：1 盘后：-2；港股可以设置3个中的任意值
        """
        request_data = {
            'security': security_param,
            'mktTmType':mkt_tm_type
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_TIME_SHARE_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_us_option_chain_expire_date(self, stock_code: str):
        """
        查询期权链到期日列表
        :param stock_code 正股代码
        """
        request_data = {
            'securityCode': stock_code
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_US_OPTION_CHAIN_EXPIRE_DATE_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def query_hq_us_option_chain_code(self, stock_code: str,
                                      expire_date: str,
                                      flag_in_out: int = None,
                                      option_type: str = None):
        """
        查询期权链Code列表
        :param stock_code 正股代码
        :param expire_date 到期日 yyyy/MM/dd
        :param flag_in_out 价格类型  hs.api.constant.OptionPriceFlag
        :param option_type 期权类型  hs.api.constant.OptionType
        """
        request_data = {
            'securityCode': stock_code,
            'expireDate': expire_date
        }
        if flag_in_out is not None:
            request_data['flagInOut'] = flag_in_out
        if option_type is not None:
            request_data['optionType'] = option_type
        return self._socket_client.post_request(request_url=HqUrl.HQ_US_OPTION_CHAIN_CODE_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def hq_subscribe(self, topic_id: int, security_list: list):
        """
        订阅行情推送
        :param topic_id 订阅/取消订阅行情推送的TopicId，hs.api.constant.HQSubscribeTopicId
        :param security_list list[SecurityParam] 证券信息
        """
        request_data = {
            'topicId': topic_id,
            'security': security_list
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_SUBSCRIBE_REQUEST_MSG_TYPE,
                                                request_data=request_data)

    def hq_unsubscribe(self, topic_id: int, security_list: list):
        """
        取消订阅行情推送
        :param topic_id 订阅/取消订阅行情推送的TopicId，hs.api.constant.HQSubscribeTopicId
        :param security_list list[SecurityParam] 证券信息
        """
        request_data = {
            'topicId': topic_id,
            'security': security_list
        }
        return self._socket_client.post_request(request_url=HqUrl.HQ_UNSUBSCRIBE_REQUEST_MSG_TYPE,
                                                request_data=request_data)
