# -*- coding: utf-8 -*-
import struct

from hs.common.common_utils import get_logger
from hs.common.pb.common.msg.Notify_pb2 import PBNotify
from hs.common.pb.common.msg.Response_pb2 import PBResponse
from hs.common.pb.common.response.CommonBoolResponse_pb2 import CommonBoolResponse
from hs.common.pb.common.response.CommonIntResponse_pb2 import CommonIntResponse
from hs.common.pb.common.response.CommonStringResponse_pb2 import CommonStringResponse
from hs.common.pb.hq.notify.BasicQotNotify_pb2 import BasicQotNotify
from hs.common.pb.hq.notify.BrokerNotify_pb2 import BrokerNotify
from hs.common.pb.hq.notify.OrderBookFullNotify_pb2 import OrderBookFullNotify
from hs.common.pb.hq.notify.TickerNotify_pb2 import TickerNotify
from hs.common.pb.trade.notify.TradeStockDeliverNotify_pb2 import TradeStockDeliverNotify
from hs.common.request_msg_header import RequestMsgHeader
from hs.common.request_msg_type_enum import RequestMsgTypeEnum

logging = get_logger(__name__)

# 包头FORMAT
MESSAGE_HEADER_FMT = "<2s1h2B2I128sBQ"
"""
    #pragma pack(push, APIProtoHeader, 1)
    struct APIProtoHeader
    {
        u8_t szHeaderFlag[2];   //2s-[2字节] 包头起始标志，固定为：“HS”
        u8_t msgType[2];        //1h-[2字节] 消息类型0：心跳 1：请求 2：响应 3：推送
        u8_t protoFmtType;      //B-[1字节] 协议格式类型，0为Protobuf格式
        u8_t protoVer;          //B-[1字节] 协议版本，用于迭代兼容, 目前填0
        u32_t serialNo;         //I-[4字节] 包序列号，用于对应请求包和回包，要求递增
        u32_t bodyLen;          //I-[4字节] 包体长度
        u8_t bodySHA1[128];     //128s-[128字节] 包体原始数据(加密后)的SHA1哈希值
        u8_t compressAlgorithm; //B-[1字节] 压缩算法，0：不压缩
        u8_t reserved[8];       //Q-[8字节] 保留8字节扩展
    };
    #pragma pack(pop, APIProtoHeader)
"""
PB_NOTIFY_CLASS_LIST = [TradeStockDeliverNotify,  # 成交推送
                        BasicQotNotify,  # 行情推送
                        BrokerNotify,
                        OrderBookFullNotify,
                        TickerNotify]
PB_RESPONSE_CLASS_LIST = [CommonBoolResponse,
                          CommonStringResponse,
                          CommonIntResponse]


def pack_request(request_msg_header: RequestMsgHeader, encrypt_payload: str) -> bytes:
    if type(encrypt_payload) is not bytes:
        encrypt_payload = bytes(encrypt_payload, 'utf-8')
    request_msg_header.body_len = len(encrypt_payload)
    logging.debug("pack_request, request_msg_header.body_len=" + str(request_msg_header.body_len))
    return struct.pack(MESSAGE_HEADER_FMT + "%ds" % request_msg_header.body_len,
                       bytes(request_msg_header.sz_header_flag, "utf-8"),
                       request_msg_header.msg_type,
                       request_msg_header.proto_fmt_type,
                       request_msg_header.proto_ver,
                       request_msg_header.serial_no,
                       request_msg_header.body_len,
                       request_msg_header.body_sha1,
                       request_msg_header.compress_algorithm,
                       request_msg_header.reserved,
                       encrypt_payload)


def unpack_response(response: bytes):
    pack_size = struct.calcsize(MESSAGE_HEADER_FMT)
    logging.debug(f"unpack_response->len(response)：{len(response)}, header pack_size：{pack_size}，response：{response}")
    unpack_msg = struct.unpack(MESSAGE_HEADER_FMT + "%ds" % (len(response) - pack_size), response)
    logging.debug(f"unpack_response->header unpack_msg：{unpack_msg}")
    msg_header = RequestMsgHeader(sz_header_flag=unpack_msg[0].decode("utf-8"),
                                  msg_type=unpack_msg[1],
                                  proto_fmt_type=unpack_msg[2],
                                  proto_ver=unpack_msg[3],
                                  serial_no=unpack_msg[4],
                                  body_len=unpack_msg[5],
                                  body_sha1=unpack_msg[6],
                                  compress_algorithm=unpack_msg[7],
                                  reserved=unpack_msg[8])
    decrypt_body_bytes = unpack_msg[9]
    pb_response = PBNotify() if (msg_header.msg_type == RequestMsgTypeEnum.NOTIFY.value) else PBResponse()
    pb_response.ParseFromString(decrypt_body_bytes)
    if msg_header.msg_type == RequestMsgTypeEnum.NOTIFY.value:
        logging.debug(f"消息解析，notifyMsgType：{pb_response.notifyMsgType}，")
    else:
        logging.debug(f"消息解析，response消息：{pb_response.responseMsg}，")
    logging.debug(f"unpack_response->response：{pb_response}")
    # payload = parse_payload(pb_response)
    return msg_header, pb_response


def parse_payload(pb_response):
    """解析any类型的payload"""
    if pb_response is None:
        return None
    payload = None
    payload_any = pb_response.payload
    if pb_response.DESCRIPTOR.name == PBNotify.DESCRIPTOR.name:
        for pb_class in PB_NOTIFY_CLASS_LIST:
            if payload_any.Is(pb_class.DESCRIPTOR):
                pb_class_response = pb_class()
                pb_response.payload.Unpack(pb_class_response)
                payload = pb_class_response
                logging.debug(f"parse_payload->{pb_class_response.DESCRIPTOR.name}：{payload}")
                break
    else:
        for pb_class in PB_RESPONSE_CLASS_LIST:
            if payload_any.Is(pb_class.DESCRIPTOR):
                pb_class_response = pb_class()
                pb_response.payload.Unpack(pb_class_response)
                payload = pb_class_response
                logging.debug(f"parse_payload->{pb_class_response.DESCRIPTOR.name}：{payload}")
                break
    return payload
