# -*- coding: utf-8 -*-
import asyncio
import errno
import json
import queue
import selectors
import socket
import ssl
import struct
import threading
import time
import traceback
import urllib.request as request
import uuid
from dataclasses import dataclass
from logging import Logger

from google.protobuf.any_pb2 import Any

from hs.common import rsa_utils, protobuf_utils
from hs.common.pb.common.constant.ResponseMsgType_pb2 import ResponseMsgType
from hs.common.pb.common.msg.HeartBeat_pb2 import HeartBeat
from hs.common.pb.common.msg.Notify_pb2 import PBNotify
from hs.common.pb.common.msg.Request_pb2 import PBRequest
from hs.common.pb.common.msg.Response_pb2 import PBResponse
from hs.common.protobuf_utils import MESSAGE_HEADER_FMT
from hs.common.request_constant import CommonUrl
from hs.common.request_msg_header import RequestMsgHeader
from hs.common.request_msg_type_enum import RequestMsgTypeEnum
from hs.common.ring_buffer import WrappedQueue

DEBUG_MODE: bool = True
BRING_NON_TARGET_WAIT_RESULT_BACK: bool = False
MAX_RECV_EMPTY_BYTE_RETRY_TIMES = 0
MAX_START_RETRY_TIMES = 1
TIMEOUT_SEC = 10
SID = "b3071fdb510b4f42800e7c32a7f43e48"
HEADERS = {
    'Accept-Charset': 'utf-8',
    'Content-Type': 'application/json'
}


@dataclass
class SyncResult:
    result: object = None


@dataclass
class ResponseResult:
    start: int = None
    response: PBResponse = None


class SocketClient(object):

    def __init__(self, ip: str, http_port: str, tcp_port: str, logger: Logger):
        self._logging = logger
        self._socket = None
        self._ring_buffer = WrappedQueue(1)  # 设置最大容量为1
        self._notify_ring_buffer = queue.Queue(10000)
        self._selector = selectors.DefaultSelector()
        self._thread = None
        self._lock = threading.RLock()
        self._notify_thread = None
        self._heartbeat_interval_sec = 10  # coming from init connection response
        self._serial_no = 0
        self._heartbeat_timer = None
        self._clear_response_result_timer = None
        self._read_buf = bytearray()
        self._is_alive = False
        self._conn_info = -1
        self._handle_notify_callback = None  # 推送回报callback
        self._max_start_retry_times = MAX_START_RETRY_TIMES
        self._max_recv_empty_byte_times = MAX_RECV_EMPTY_BYTE_RETRY_TIMES  # 连续收到b''的最大次数
        self.g_serial_no = 0
        self.g_serial_no_lock = threading.RLock()
        self._ip = ip
        self._http_port = http_port
        self._tcp_port = tcp_port
        self._domain = CommonUrl.HTTP_PROTOCOL_PREFIX + ip + CommonUrl.COLON + str(http_port)

    def post_request(self, request_url: str, request_data: dict) -> str:
        # request param
        try:
            # request url
            request_url = self._domain + request_url
            param_data = { 
                'timeout_sec': TIMEOUT_SEC,
                'params': request_data
            }
            start = int(round(time.time() * 1000))
            encode_data = json.dumps(param_data)
            self._logging.info(f"Post request param data：{encode_data}")
            req = request.Request(url=request_url, data=encode_data.encode("utf-8"), headers=HEADERS, method='POST')
            with request.urlopen(req, context=ssl._create_unverified_context(), timeout=10) as resp:
                response = resp.read()
            response = json.loads(rsa_utils.bytes_to_str(response))

            cost_time = int(round(time.time() * 1000)) - start
            self._logging.info(f"Post request url：{request_url} cost: {cost_time}ms, response: {response}")
            return response
        except Exception as e:
            self._logging.error(f"Post request url: {request_url}, param: {param_data}, error：{e}")

    def restart(self):
        self._logging.info(f"socket client start reconnect: {self._ip}:{self._tcp_port}.")
        self.stop()
        self.start()
        if self.is_alive():
            self._logging.info(f"Socket client reconnect: {self._ip}:{self._tcp_port} success.")
        else:
            self._logging.info(f"Socket client reconnect: {self._ip}:{self._tcp_port} fail.")

    def start(self):
        """
        Should be called from main thread
        :return:
        """
        self._ring_buffer.notify_all()  # notify all ringbuffer waiter
        self._logging.debug("Starting the socket client.")
        self._serial_no = self.init_serial_no(10)

        self._conn_info = self.socket_connect()
        self.register()
        # poll thread
        self._is_alive = True
        with self._lock:
            if self._thread is None:
                self._thread = threading.Thread(target=self._thread_selector)
                self._thread.setDaemon(False)  # default False
                self._thread.start()

        # add notify callback
        if self._handle_notify_callback is not None and callable(self._handle_notify_callback):
            self._logging.info("设置消息推送回调 handle_notify_callback")
            self.handle_notify_for_ever(self._handle_notify_callback)
        # check keep alive
        self.heartbeat(self._heartbeat_interval_sec)
        self._max_start_retry_times = MAX_START_RETRY_TIMES
        self._max_recv_empty_byte_times = MAX_RECV_EMPTY_BYTE_RETRY_TIMES
        self._logging.info("The socket client is started!")

    def stop(self):
        self.unregister()
        self._is_alive = False
        if self._heartbeat_timer:
            self._heartbeat_timer.cancel()
            self._heartbeat_timer = None
        if self._clear_response_result_timer:
            self._clear_response_result_timer.cancel()
            self._clear_response_result_timer = None
        self._ring_buffer.clear()
        self._notify_ring_buffer.put(None)
        self._thread = None
        self._notify_thread = None
        self._serial_no = self.init_serial_no(10)
        self._read_buf[:] = b'\x00' * len(self._read_buf)
        self._socket = None
        # signal.signal(signal.SIGCHLD, grim_reaper)
        self._ring_buffer.notify_all()  # notify all ringbuffer waiter
        self._logging.info("The socket client is stopped!")

    def register(self):
        if self._socket and not self._socket._closed:
            self._selector.register(self._socket, selectors.EVENT_READ, data=self)
            self._logging.debug("selector registered!")

    def unregister(self):
        for sel_key in list(self._selector.get_map().values()):
            self._selector.unregister(sel_key.fileobj)
            sel_key.fileobj.close()

    def heartbeat(self, interval=10):
        if not self.is_alive():
            self._logging.debug(f"[KeepAlive] socket is not alive, ignore this heartbeat!")
            return "", 0
        # build payload
        payload = HeartBeat()
        request_id, msg_bytes, sent_bytes_len = \
            self.build_request_bytes_then_send(request_msg_type=0, msg_header_type_enum=RequestMsgTypeEnum.HEART_BEAT,
                                               pb_payload=payload)
        if self._heartbeat_timer:
            self._heartbeat_timer.cancel()
            self._heartbeat_timer = None
        self._heartbeat_timer = threading.Timer(interval, self.heartbeat, (interval,))
        self._heartbeat_timer.start()
        self._logging.debug(f"[KeepAlive] sent a heartbeat message {request_id}, once every {interval} seconds")
        return request_id, sent_bytes_len

    def build_request_bytes_then_send(self, request_msg_type: int, msg_header_type_enum: RequestMsgTypeEnum,
                                      pb_payload):
        if not self.is_alive():
            self._logging.error(
                f"Socket is not alive, request_msg_type：{request_msg_type}, msg_header_type_enum：{msg_header_type_enum}, ignore this request.")
            return None, None, 0
        try:
            request = PBRequest()
            request.requestMsgType = request_msg_type
            request.token = SID
            request.requestId = str(uuid.uuid1())
            request.requestTime = int(round(time.time() * 1000))
            # build request message header
            request_msg_header = RequestMsgHeader()
            request_msg_header.serial_no = self.compare_and_set_serial_no(self._serial_no)
            request_msg_header.msg_type = RequestMsgTypeEnum.get_id(msg_header_type_enum)
            # set request payload
            if pb_payload:
                any_obj = Any()
                any_obj.Pack(pb_payload)
                request.payload.CopyFrom(any_obj)
            # set request header's body_sha1
            request_msg_header.body_sha1 = bytes(SID, encoding='utf-8')
            # 把数据对象打包为字节对象
            encrypt_payload = request.SerializeToString()
            msg_bytes = protobuf_utils.pack_request(request_msg_header, encrypt_payload)
            # send msg
            sent_bytes_len = self._socket.send(msg_bytes)
            return request.requestId, msg_bytes, sent_bytes_len
        except IOError as e:
            self._is_alive = False
            if DEBUG_MODE:
                self._logging.error(traceback.format_exc())
            else:
                self._logging.error(f"IOError Occurred：{type(e)}, maybe need restart the socket!")
            if e.errno == errno.EPIPE:
                self._logging.error(f"BrokenPipeError Occurred：{type(e)}, currently maybe need restart the socket!")
        except Exception as e:
            self._is_alive = False
            if DEBUG_MODE:
                self._logging.error(traceback.format_exc())
            else:
                self._logging.error(f"Send error：{type(e)}, maybe need restart the socket!")
        return None, None, 0

    def socket_connect(self):
        """
        connection and return connection info
        return: 0-OK，not 0-fail
        """
        try:
            self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

            # FIX问题：python - Python，为什么在使用TCP套接字时出现10035(在服务器上)和10053(在客户端上)错误？ https://www.coder.work/article/386867
            self._socket.setblocking(True)
            conn_info = self._socket.connect_ex((self._ip, self._tcp_port))
            self._socket.setblocking(False)  # 将setblocking(False)放到connect方法之后执行
            # self._socket.settimeout(0.02)  # non-blocking。时间久了就可能在在recv()处出现异常 ConnectionResetError: [Errno 54] Connection reset by peer
            self._logging.info(f"Socket connect info：{conn_info}, socket.fileno()：{self._socket.fileno()}")
            return conn_info
        except Exception as e:
            self.stop()
            self._logging.error(traceback.format_exc())
        return -1

    def handle_notify_for_ever(self, callback):
        self._handle_notify_callback = callback

        def wait_notify(socket_client_self):
            while True:
                if socket_client_self._notify_thread is None:
                    break
                try:
                    # default block=True
                    pb_notify = socket_client_self._notify_ring_buffer.get()
                except queue.Empty:
                    continue
                if pb_notify is None:
                    continue
                self._logging.info(
                    f"Notify msg，notifyId: {pb_notify.notifyId}, notifyMsgType：{pb_notify.notifyMsgType}")
                if callable(socket_client_self._handle_notify_callback):
                    socket_client_self._handle_notify_callback(pb_notify)
                else:
                    self._logging.error(f"Notify callback is not callable：{socket_client_self._handle_notify_callback}")
                    # break 不跳出循环，持续消费

        if self._notify_thread is None:
            self._notify_thread = threading.Thread(target=wait_notify, args=(self,))
            self._notify_thread.setDaemon(True)
            self._notify_thread.start()

    async def wait_result(self, request_id=None,
                          future=None,
                          callback=None,
                          asyncio_semaphore: asyncio.Semaphore = asyncio.Semaphore(1)):
        async with asyncio_semaphore:
            while True:
                try:
                    if len(self._ring_buffer) < 1 and self.is_alive():
                        await asyncio.sleep(0.02)
                        continue
                    pb_response = self._ring_buffer.popleft()
                    if request_id and future:
                        if pb_response.requestId == request_id:
                            future.set_result(pb_response)
                            return
                        elif BRING_NON_TARGET_WAIT_RESULT_BACK:
                            # 如果不是本次指定的request_id对应的结果，就放回ring_buffer的最左边元素位置
                            self._ring_buffer.appendleft(pb_response)
                            self._logging.error(
                                f"request_id：{request_id} got other's result, now put it back to ringbuffer")
                        else:
                            self._logging.error(
                                f"request_id：{request_id} got result of another request_id：{pb_response.requestId}, ignore it!")
                    if callable(callback):
                        callback(pb_response)
                    else:
                        self._logging.error(f"Callback is not callable：{callback}")
                    # finally, return None future
                    future.set_result(None)
                    return
                except Exception as e:
                    self._logging.info(f"Error Occurred：{e}，currently release the wait_result!")
                    future.set_result(None)
                    return

    def _thread_selector(self):
        while True:
            with self._lock:
                if not self.is_alive():
                    self._thread = None
                    break
            self.poll()

    def poll(self):
        events = self._selector.select(0.001)  # select(0.02)
        for key, evt_mask in events:
            conn = key.data
            #######
            # read_socket, write_socket = socket.socketpair()
            # print(f"[DDDDDDDDDEBUG]：{key.fileobj==read_socket}")
            #######
            if evt_mask & selectors.EVENT_READ != 0:
                try:
                    if not self.is_alive():
                        self._logging.info("Socket connection not alive.")
                        break
                    # 在socket非阻塞模式下，如果调用recv()没有发现任何数据，或send()调用无法立即发送数据，那么将引起socket.error异常。
                    # 参考：socket recv 10035 error 如何解决 https://bbs.csdn.net/topics/210068263
                    self.decode_response(conn)
                except socket.error as e:
                    if DEBUG_MODE:
                        self._logging.error(traceback.format_exc())
                    else:
                        self._logging.error(f"BlockingIOError Occurred：{type(e)}, maybe need restart the socket!")
                    if e.errno == errno.EAGAIN:  # str(e) == "[Errno 35] Resource temporarily unavailable"
                        self._logging.error(f"BlockingIOError Occurred：{type(e)}, just need continue to next step!")
                        # time.sleep(0.1)  # just need continue to next step
                    self.stop()  # 解决Windows下报错：ConnectionResetError: [WinError 10054] 远程主机强迫关闭了一个现有的连接。
                except Exception as e:
                    if DEBUG_MODE:
                        self._logging.error(traceback.format_exc())
                    else:
                        self._logging.error(f"Poll decode_response Error：{e}")
                    self.stop()  # 解决Windows下报错：ConnectionResetError: [WinError 10054] 远程主机强迫关闭了一个现有的连接。

    def decode_response(self, conn):
        # 数据包解码
        recv_bytes = self._socket.recv(1024 * 1024)  # buffer size e.g. 151 或 1024 或 128*1024

        if recv_bytes == b'':  # recv超时异常：TimeoutError: [Errno 60] Operation timed out 断网很久就会连续发送b''，如果次数较多就需要人工重启连接
            self._max_recv_empty_byte_times -= 1
            self._logging.error(
                f"Note: The decode_response recv_bytes == b''，max_recv_empty_byte_times：{self._max_recv_empty_byte_times}!")
            if self._max_recv_empty_byte_times < 0:
                # 断网太久，期间有send操作导致超时，可尝试方案：socket.setdefaulttimeout(10)、sock.settimeout(20|None) 或 sock.setblocking(True|False)
                self._is_alive = False
                self.stop()
                return
        else:
            self._max_recv_empty_byte_times = MAX_RECV_EMPTY_BYTE_RETRY_TIMES
            self._read_buf.extend(recv_bytes)

        while len(self._read_buf) > 0:
            # 检查readbuf是否可以完整解1个包
            pack_size = struct.calcsize(MESSAGE_HEADER_FMT)
            if len(self._read_buf) < pack_size:
                return
            unpack_msg = struct.unpack(MESSAGE_HEADER_FMT, self._read_buf[:pack_size])
            body_len = unpack_msg[5]
            if len(self._read_buf) < pack_size + body_len:
                self._logging.debug(
                    f"maybe the total read bytes({len(self._read_buf)}) is not enough to {pack_size}+{body_len}, just waiting the next bytes streaming!")
                return
            full_pack_bytes = self._read_buf[:pack_size + body_len]
            del self._read_buf[:pack_size + body_len]

            # 数据包解析
            msg_header, pb_response = protobuf_utils.unpack_response(full_pack_bytes)

            if msg_header.msg_type == RequestMsgTypeEnum.HEART_BEAT.value:
                self._logging.debug(
                    f"[KeepAlive] got response for heartbeat request, self._last_keep_alive_seconds_int：{self._last_keep_alive_seconds_int}，ignore it!")
                return
            elif msg_header.body_len < 1:
                self._logging.debug("Response payload body len < 1, ignore it!")
                return
            elif pb_response and hasattr(pb_response, "responseMsgType") and (
                    pb_response.responseMsgType == ResponseMsgType.TradeKeepLoginResponseMsgType):
                self._logging.debug(f"Got response for trading logon heartbeat, ignore it!")
                return
            self._logging.debug(
                f"Decode response total recv_bytes_len：{len(full_pack_bytes)}，request_msg_header.serial_no：{msg_header.serial_no}")

            if pb_response.DESCRIPTOR.name == PBNotify.DESCRIPTOR.name:
                # Message Notify
                self._logging.debug(
                    f"Received notify and append to notify ringbuffer, notifyType: {pb_response.notifyMsgType}, notifyId：{pb_response.notifyId}")
                self.handle_notify(pb_response)
            else:
                request_id = pb_response.requestId
                # 接口响应结果
                self._logging.debug(f"received message and append to ringbuffer, requestId：{request_id}")
                response_result = ResponseResult()
                response_result.response = pb_response
                response_result.start = int(round(time.time()))

    def handle_notify(self, pb_notify):
        self._notify_ring_buffer.put(pb_notify)

    def is_alive(self) -> bool:
        alive = self._is_alive and not (self._socket is None or self._socket._closed)
        return alive

    def init_serial_no(self, serial_no):
        with self.g_serial_no_lock:
            self.g_serial_no = serial_no
        return self.g_serial_no

    def compare_and_set_serial_no(self, serial_no):
        with self.g_serial_no_lock:
            ret_id = self.g_serial_no
            self.g_serial_no += 1
            # 4294967295是2的32次方-1
            if self.g_serial_no >= 4294967295:
                self.g_serial_no = 10
            return ret_id

    def __enter__(self):
        if not self.is_alive():
            self._logging.info("Socket connection is not alive or init connection not done, ready to reconnect.")
            self.restart()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type is not None and exc_val is not None and exc_tb is not None:
            # exit with-as once catching the throwed except in with-as body
            self.stop()
            traceback.print_exception(exc_type, exc_val, exc_tb)
            return True
        return False
