# -*- coding: utf-8 -*-
import base64
from hs.common.aes_ecb import AESCipher

from hs.common.common_utils import now_to_str

from hs.api.hq_constant import DataType, MktTmType, Direction, ExRightFlag, CycType, OptionPriceFlag, \
    OptionType, HQSubscribeTopicId
from hs.api.futures_constant import FuturesEntrustType, FuturesValidTimeType

from hs.api.hs_open_api import OpenAPI
from hs.api.trade_constant import RateType, ExchangeType, EntrustType, SessionType, EntrustBS, EntrustEX
from hs.common.pb.common.constant.NotifyMsgType_pb2 import TradeStockDeliverMsgType, FuturesTradeStockDeliverMsgType, \
    BasicQotNotifyMsgType, BrokerQueueNotifyMsgType, OrderBookNotifyMsgType, TickerNotifyMsgType
from hs.common.protobuf_utils import parse_payload


class OpenAPIDemo(object):
    """股票交易类接口独立Demo程序"""

    def __init__(self, **kwargs):
        self._open_api = OpenAPI(**kwargs)
        self._timer_threading = None

    def check_alive(self) -> bool:
        """检查Trading API是否正常连接状态"""
        return self._open_api.is_alive()

    def start(self):
        """启动业务API上下文环境，重启StockClient"""
        self._open_api.start()
        return self

    def add_notify_callback(self):
        """增加消息推送回调函数"""
        self._open_api.add_notify_callback(self.notify_callback)
        return self

    ##########################################      Open API      ##################################################

    def query_rate_list(self):
        """查询汇率列表"""
        response = self._open_api.query_rate_list(rate_type=RateType.IMMEDIATE)
        if response is not None and response.get('ok'):
            print(f"查询汇率列表：{response.get('data')}")
        else:
            print(f"Fail to query_rate_list, response: {response}")

    def trade_login(self):
        """交易登录"""

        # 交易密码需要使用AES加密（ECB/PKCSPadding），AES秘钥为"m+qS04/2CH1OweCnmXZ3TDZkCQS+hBzY"，需要进行Base64处理
        # 加密步骤：Base64.Encode(AES.Encrypt(Base64.Decode("m+qS04/2CH1OweCnmXZ3TDZkCQS+hBzY"), "123456"))
        # 如密码为"123456"，加密后密码："W1U8iZIppSE+mBMtzy9vZQ=="
        password = "123456"
        aes_key_base64 = "m+qS04/2CH1OweCnmXZ3TDZkCQS+hBzY"
        aes_key = base64.b64decode(aes_key_base64)
        pwd_encrypted = AESCipher(aes_key).encrypt(password)
        pwd_encrypted_base64 = base64.b64encode(pwd_encrypted).decode("utf-8")

        response = self._open_api.trade_login(password=pwd_encrypted_base64)
        if response is not None and response.get('ok'):
            print(f"交易登录成功：{response.get('data')}")
        else:
            print(f"Fail to trade_login, response: {response}")

    def trade_logout(self):
        """交易登出"""
        response = self._open_api.trade_logout()
        if response is not None and response.get('ok'):
            print(f"交易登出成功：{response.get('data')}")
        else:
            print(f"Fail to trade_logout, response: {response}")

    def query_margin_fund_info(self):
        """查询客户资金信息"""
        response = self._open_api.query_margin_fund_info(exchange_type=ExchangeType.K)
        if response is not None and response.get('ok'):
            print(f"查询客户资金信息：{response.get('data')}")
        else:
            print(f"Fail to query_margin_fund_info, response: {response}")

    def query_holds_list(self):
        """查询持仓列表"""
        response = self._open_api.query_holds_list(exchange_type=ExchangeType.K)
        if response is not None and response.get('ok'):
            print(f"查询持仓列表：{response.get('data')}")
        else:
            print(f"Fail to query_holds_list, response: {response}")

    def query_buy_amount(self):
        """查询最大可买"""
        response = self._open_api.query_buy_amount(exchange_type=ExchangeType.K,
                                                   stock_code="00700.HK",
                                                   entrust_price="313.00")
        if response is not None and response.get('ok'):
            print(f"查询最大可买：{response.get('data')}")
        else:
            print(f"Fail to query_buy_amount, response: {response}")

    def query_sell_amount(self):
        """查询最大可卖"""
        response = self._open_api.query_sell_amount(exchange_type=ExchangeType.K, stock_code="00700.HK")
        if response is not None and response.get('ok'):
            print(f"查询最大可卖：{response.get('data')}")
        else:
            print(f"Fail to query_sell_amount, response: {response}")

    def query_real_fund_jour_list(self):
        """查询当日资金流水"""
        response = self._open_api.query_real_fund_jour_list(exchange_type=ExchangeType.K,
                                                            query_count=20,
                                                            query_param_str="0")
        if response is not None and response.get('ok'):
            print(f"查询当日资金流水：{response.get('data')}")
        else:
            print(f"Fail to query_real_fund_jour_list, response: {response}")

    def query_history_fund_jour_list(self):
        """查询历史资金流水"""
        response = self._open_api.query_history_fund_jour_list(exchange_type=ExchangeType.K,
                                                               query_count=20,
                                                               query_param_str="0",
                                                               start_date="20201210",
                                                               end_date=now_to_str())
        if response is not None and response.get('ok'):
            print(f"查询历史资金流水：{response.get('data')}")
        else:
            print(f"Fail to query_history_fund_jour_list, response: {response}")

    def entrust(self):
        """下单"""
        response = self._open_api.entrust(exchange_type=ExchangeType.K,
                                          stock_code="00700.HK",
                                          session_type=SessionType.GENERAL_ORDER_NO,
                                          entrust_price="313",
                                          entrust_amount="100",
                                          entrust_type=EntrustType.LIMIT,
                                          entrust_bs=EntrustBS.BUY,
                                          exchange=EntrustEX.ARCA,
                                          iceberg_display_size="0")
        if response is not None and response.get('ok'):
            print(f"下单：{response.get('data')}")
        else:
            print(f"Fail to entrust, response: {response}")

    def cancel_entrust(self):
        """撤单"""
        response = self._open_api.cancel_entrust(exchange_type=ExchangeType.K,
                                                 entrust_id="5633414",
                                                 stock_code="00700.HK")
        if response is not None and response.get('ok'):
            print(f"撤单：{response.get('data')}")
        else:
            print(f"Fail to cancel_entrust, response: {response}")

    def batch_cancel_entrust(self):
        """批量撤单"""
        response = self._open_api.batch_cancel_entrust(exchange_type=ExchangeType.K, entrust_ids=["5633414"])
        if response is not None and response.get('ok'):
            print(f"批量撤单：{response.get('data')}")
        else:
            print(f"Fail to batch_cancel_entrust, response: {response}")

    def change_entrust(self):
        """改单"""
        response = self._open_api.change_entrust(exchange_type=ExchangeType.K,
                                                 entrust_id="5633414",
                                                 stock_code="00700.HK",
                                                 entrust_amount="100",
                                                 entrust_price="315")
        if response is not None and response.get('ok'):
            print(f"改单：{response.get('data')}")
        else:
            print(f"Fail to change_entrust, response: {response}")

    def query_max_available_asset(self):
        """查询最大可用资产"""
        response = self._open_api.query_max_available_asset(exchange_type=ExchangeType.K,
                                                            stock_code="00700.HK",
                                                            entrust_price="313",
                                                            entrust_type=EntrustType.MARKET)
        if response is not None and response.get('ok'):
            print(f"查询最大可用资产：{response.get('data')}")
        else:
            print(f"Fail to query_max_available_asset, response: {response}")

    def query_real_entrust_list(self):
        """查询当日委托"""
        response = self._open_api.query_real_entrust_list(exchange_type=ExchangeType.K,
                                                          query_count=20,
                                                          query_param_str="0")
        if response is not None and response.get('ok'):
            print(f"查询当日委托：{response.get('data')}")
        else:
            print(f"Fail to query_real_entrust_list, response: {response}")

    def query_history_entrust_list(self):
        """查询历史委托"""
        response = self._open_api.query_history_entrust_list(exchange_type=ExchangeType.K,
                                                             query_count=20,
                                                             query_param_str="0",
                                                             start_date="20230101",
                                                             end_date=now_to_str())
        if response is not None and response.get('ok'):
            print(f"查询历史委托：{response.get('data')}")
        else:
            print(f"Fail to query_history_entrust_list, response: {response}")

    def query_real_deliver_list(self):
        """查询当日成交"""
        response = self._open_api.query_real_deliver_list(exchange_type=ExchangeType.K,
                                                          query_count=20,
                                                          query_param_str="0")
        if response is not None and response.get('ok'):
            print(f"查询当日成交：{response.get('data')}")
        else:
            print(f"Fail to query_real_deliver_list, response: {response}")

    def query_history_deliver_list(self):
        """查询历史成交"""
        response = self._open_api.query_history_deliver_list(exchange_type=ExchangeType.K,
                                                             query_count=20,
                                                             query_param_str="0",
                                                             start_date="20230101",
                                                             end_date=now_to_str())
        if response is not None and response.get('ok'):
            print(f"查询历史成交：{response.get('data')}")
        else:
            print(f"Fail to query_history_deliver_list, response: {response}")

    def query_real_cond_order_list(self):
        """查询当日条件单"""
        response = self._open_api.query_real_cond_order_list(exchange_type=ExchangeType.K,
                                                             stock_code="",
                                                             page_no=1,
                                                             page_size=20)
        if response is not None and response.get('ok'):
            print(f"查询当日条件单：{response.get('data')}")
        else:
            print(f"Fail to query_real_cond_order_list, response: {response}")

    def query_history_cond_order_list(self):
        """查询历史条件单"""
        response = self._open_api.query_history_cond_order_list(start_time="2023-01-01 19:21:21",
                                                                end_time="2023-01-02 19:21:21",
                                                                exchange_type=ExchangeType.K,
                                                                stock_code="",
                                                                page_no=1,
                                                                page_size=20)
        if response is not None and response.get('ok'):
            print(f"查询历史条件单：{response.get('data')}")
        else:
            print(f"Fail to query_history_cond_order_list, response: {response}")

    def query_before_and_after_support(self):
        """查询是否支持盘前盘后交易"""
        response = self._open_api.query_before_and_after_support(exchange_type=ExchangeType.P,
                                                                 stock_code="TSLA")
        if response is not None and response.get('ok'):
            print(f"查询是否支持盘前盘后交易：{response.get('data')}")
        else:
            print(f"Fail to query_before_and_after_support, response: {response}")

    def query_stock_short_info(self):
        """查询股票沽空信息"""
        response = self._open_api.query_stock_short_info(exchange_type=ExchangeType.P,
                                                         stock_code="AAPL")
        if response is not None and response.get('ok'):
            print(f"查询股票沽空信息：{response.get('data')}")
        else:
            print(f"Fail to query_stock_short_info, response: {response}")

    def query_margin_full_info(self):
        """查询股票融资融券数据"""
        response = self._open_api.query_margin_full_info(data_type=str(DataType.US_STOCK),
                                                         stock_code="AAPL")
        if response is not None and response.get('ok'):
            print(f"查询股票融资融券数据：{response.get('data')}")
        else:
            print(f"Fail to query_margin_full_info, response: {response}")

    def futures_query_fund_info(self):
        """期货-查询客户资金信息"""
        response = self._open_api.futures_query_fund_info()
        if response is not None and response.get('ok'):
            print(f"期货-查询客户资金信息：{response.get('data')}")
        else:
            print(f"Fail to futures_query_fund_info, response: {response}")

    def futures_query_holds_list(self):
        """期货-查询持仓列表"""
        response = self._open_api.futures_query_holds_list()
        if response is not None and response.get('ok'):
            print(f"期货-查询持仓列表：{response.get('data')}")
        else:
            print(f"Fail to futures_query_holds_list, response: {response}")

    def futures_query_max_buy_sell_amount(self):
        """期货-查询最大可买可卖"""
        response = self._open_api.futures_query_max_buy_sell_amount(stock_code="CUSH2")
        if response is not None and response.get('ok'):
            print(f"期货-查询最大可买可卖：{response.get('data')}")
        else:
            print(f"Fail to futures_query_max_buy_sell_amount, response: {response}")

    def futures_query_product_info(self):
        """期货-查询合约产品信息"""
        response = self._open_api.futures_query_product_info(stock_code_list=["CUSH2"])
        if response is not None and response.get('ok'):
            print(f"期货-查询合约产品信息：{response.get('data')}")
        else:
            print(f"Fail to futures_query_product_info, response: {response}")

    def futures_entrust(self):
        """期货-委托下单"""
        response = self._open_api.futures_entrust(stock_code="CUSH2",
                                                  entrust_type=FuturesEntrustType.LIMIT,
                                                  entrust_price="5.5",
                                                  entrust_amount="100",
                                                  entrust_bs=EntrustBS.BUY,
                                                  valid_time_type=FuturesValidTimeType.VALID_ON_SPECIFIED_DATE,
                                                  valid_time="20221230")
        if response is not None and response.get('ok'):
            print(f"期货-委托下单：{response.get('data')}")
        else:
            print(f"Fail to futures_entrust, response: {response}")

    def futures_cancel_entrust(self):
        """交易接口-期货-委托撤单"""
        response = self._open_api.futures_cancel_entrust(entrust_id="1234567",
                                                         stock_code="CUSH2")
        if response is not None and response.get('ok'):
            print(f"期货-委托撤单：{response.get('data')}")
        else:
            print(f"Fail to futures_cancel_entrust, response: {response}")

    def futures_change_entrust(self):
        """交易接口-期货-委托改单"""
        response = self._open_api.futures_change_entrust(entrust_id="1234567",
                                                         stock_code="CUSH2",
                                                         entrust_price="5.5",
                                                         entrust_amount="100",
                                                         entrust_bs=EntrustBS.BUY,
                                                         valid_time_type=FuturesValidTimeType.VALID_ON_SPECIFIED_DATE,
                                                         valid_time="20221230")
        if response is not None and response.get('ok'):
            print(f"期货-委托改单：{response.get('data')}")
        else:
            print(f"Fail to futures_entrust, response: {response}")

    def futures_query_real_entrust_list(self):
        """ 交易接口-期货-查询今日委托"""
        response = self._open_api.futures_query_real_entrust_list()
        if response is not None and response.get('ok'):
            print(f"期货-查询今日委托：{response.get('data')}")
        else:
            print(f"Fail to futures_query_real_entrust_list, response: {response}")

    def futures_query_history_entrust_list(self):
        """交易接口-期货-查询历史委托"""
        response = self._open_api.futures_query_history_entrust_list(page_no=1,
                                                                     page_size=20,
                                                                     start_date="20211220",
                                                                     end_date=now_to_str())
        if response is not None and response.get('ok'):
            print(f"期货-查询历史委托：{response.get('data')}")
        else:
            print(f"Fail to futures_query_history_entrust_list, response: {response}")

    def futures_query_real_deliver_list(self):
        """交易接口-期货-查询今日成交"""
        response = self._open_api.futures_query_real_deliver_list()
        if response is not None and response.get('ok'):
            print(f"期货-查询今日成交：{response.get('data')}")
        else:
            print(f"Fail to futures_query_real_deliver_list, response: {response}")

    def futures_query_history_deliver_list(self):
        """交易接口-期货-查询历史成交"""
        response = self._open_api.futures_query_history_deliver_list(page_no=1,
                                                                     page_size=20,
                                                                     start_date="20211220",
                                                                     end_date=now_to_str())
        if response is not None and response.get('ok'):
            print(f"期货-查询历史成交：{response.get('data')}")
        else:
            print(f"Fail to futures_query_history_deliver_list, response: {response}")

    def trade_subscribe(self):
        """订阅交易推送消息"""
        response = self._open_api.trade_subscribe()
        if response is not None and response.get('ok'):
            print(f"订阅交易推送消息：{response.get('data')}")
        else:
            print(f"Fail to subscribe, response: {response}")

    def trade_unsubscribe(self):
        """取消订阅交易推送消息"""
        response = self._open_api.trade_unsubscribe()
        if response is not None and response.get('ok'):
            print(f"取消订阅交易推送消息：{response.get('data')}")
        else:
            print(f"Fail to unsubscribe, response: {response}")

    def query_hq_basic_qot(self):
        """查询基础报价"""
        response = self._open_api.query_hq_basic_qot(
            security_list=[{"dataType": DataType.US_STOCK, "code": "TSLA"}],
            mkt_tm_type=MktTmType.MID_SESSION)
        if response is not None and response.get('ok'):
            print(f"查询基础报价：{response.get('data')}")
        else:
            print(f"Fail to query_hq_basic_qot, response: {response}")

    def query_hq_broker(self):
        """查询买卖经纪摆盘"""
        response = self._open_api.query_hq_broker(security_param={"dataType": DataType.HK_STOCK, "code": "00700.HK"})
        if response is not None and response.get('ok'):
            print(f"查询买卖经纪摆盘：{response.get('data')}")
        else:
            print(f"Fail to query_hq_broker, response: {response}")

    def query_hq_order_book(self):
        """查询买卖档"""
        response = self._open_api.query_hq_order_book(security_param={"dataType": DataType.US_STOCK, "code": "TSLA"},
                                                      mkt_tm_type=MktTmType.MID_SESSION)
        if response is not None and response.get('ok'):
            print(f"查询买卖档：{response.get('data')}")
        else:
            print(f"Fail to query_hq_order_book, response: {response}")

    def query_hq_ticker(self):
        """查询逐笔"""
        response = self._open_api.query_hq_ticker(security_param={"dataType": DataType.US_STOCK, "code": "TSLA"},
                                                  limit=10,
                                                  mkt_tm_type=MktTmType.MID_SESSION)
        if response is not None and response.get('ok'):
            print(f"查询逐笔：{response.get('data')}")
        else:
            print(f"Fail to query_hq_ticker, response: {response}")

    def query_hq_kline(self):
        """查询K线"""
        response = self._open_api.query_hq_kline(security_param={"dataType": DataType.HK_STOCK, "code": "00700.HK"},
                                                 start_date="20230903",
                                                 direction=Direction.QUERY_LEFT,
                                                 ex_right_flag=ExRightFlag.BFQ,
                                                 cyc_type=CycType.DAY,
                                                 limit=10)
        if response is not None and response.get('ok'):
            print(f"查询K线：{response.get('data')}")
        else:
            print(f"Fail to query_hq_kline, response: {response}")

    def query_hq_time_share(self):
        """查询分时数据"""
        response = self._open_api.query_hq_time_share(security_param={"dataType": DataType.US_STOCK, "code": "TSLA"},
                                                      mkt_tm_type=MktTmType.MID_SESSION)
        if response is not None and response.get('ok'):
            print(f"查询分时数据：{response.get('data')}")
        else:
            print(f"Fail to query_hq_time_share, response: {response}")

    def query_hq_us_option_chain_expire_date(self):
        """查询期权链到期日列表"""
        response = self._open_api.query_hq_us_option_chain_expire_date(stock_code="AAPL")
        if response is not None and response.get('ok'):
            print(f"查询期权链到期日列表：{response.get('data')}")
        else:
            print(f"Fail to query_hq_us_option_chain_expire_date, response: {response}")

    def query_hq_us_option_chain_code(self):
        """查询期权链Code列表"""
        response = self._open_api.query_hq_us_option_chain_code(stock_code="AAPL",
                                                                expire_date="2023/10/24",
                                                                flag_in_out=OptionPriceFlag.PRICE_IN,
                                                                option_type=OptionType.CALL)
        if response is not None and response.get('ok'):
            print(f"查询期权链Code列表：{response.get('data')}")
        else:
            print(f"Fail to query_hq_us_option_chain_code, response: {response}")

    def hq_subscribe_basic_qot(self):
        """订阅行情基础报价推送消息"""
        response = self._open_api.hq_subscribe(topic_id=HQSubscribeTopicId.BASIC_QOT,
                                               security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"订阅行情基础报价推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_subscribe_basic_qot, response: {response}")

    def hq_subscribe_broker(self):
        """订阅行情买卖经纪推送消息"""
        response = self._open_api.hq_subscribe(topic_id=HQSubscribeTopicId.BROKER,
                                               security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"订阅行情买卖经纪推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_subscribe_broker, response: {response}")

    def hq_subscribe_ticker(self):
        """订阅行情逐笔推送消息"""
        response = self._open_api.hq_subscribe(topic_id=HQSubscribeTopicId.TICKER,
                                               security_list=[{"dataType": DataType.US_STOCK, "code": "TSLA"}])
        if response is not None and response.get('ok'):
            print(f"订阅行情逐笔推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_subscribe_ticker, response: {response}")

    def hq_subscribe_order_book(self):
        """订阅买卖档推送消息"""
        response = self._open_api.hq_subscribe(topic_id=HQSubscribeTopicId.ORDER_BOOK,
                                               security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"订阅买卖档推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_subscribe_ticker, response: {response}")

    def hq_subscribe_arca_book(self):
        """订阅行情深度摆盘-ARCABOOK买卖档推送消息"""
        response = self._open_api.hq_subscribe(topic_id=HQSubscribeTopicId.ARCA_BOOK,
                                               security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"订阅行情深度摆盘-ARCABOOK买卖档推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_subscribe_arca_book, response: {response}")

    def hq_subscribe_total_view_book(self):
        """订阅行情深度摆盘-TOTALVIEW买卖档推送消息"""
        response = self._open_api.hq_subscribe(topic_id=HQSubscribeTopicId.TOTALVIEW_BOOK,
                                               security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"订阅行情深度摆盘-TOTALVIEW买卖档推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_subscribe_total_view_book, response: {response}")

    def hq_unsubscribe_basic_qot(self):
        """取消订阅行情基础报价推送消息"""
        response = self._open_api.hq_unsubscribe(topic_id=HQSubscribeTopicId.BASIC_QOT,
                                                 security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"取消订阅行情基础报价推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_unsubscribe_basic_qot, response: {response}")

    def hq_unsubscribe_broker(self):
        """取消订阅行情买卖经纪推送消息"""
        response = self._open_api.hq_unsubscribe(topic_id=HQSubscribeTopicId.BROKER,
                                                 security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"取消订阅行情买卖经纪推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_unsubscribe_broker, response: {response}")

    def hq_unsubscribe_order_book(self):
        """取消订阅行情买卖档推送消息"""
        response = self._open_api.hq_unsubscribe(topic_id=HQSubscribeTopicId.ORDER_BOOK,
                                                 security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"取消订阅行情买卖档推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_unsubscribe_order_book, response: {response}")

    def hq_unsubscribe_ticker(self):
        """取消订阅行情逐笔推送消息"""
        response = self._open_api.hq_unsubscribe(topic_id=HQSubscribeTopicId.TICKER,
                                                 security_list=[{"dataType": DataType.US_STOCK, "code": "TSLA"}])
        if response is not None and response.get('ok'):
            print(f"取消订阅行情逐笔推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_unsubscribe_ticker, response: {response}")

    def hq_unsubscribe_arca_book(self):
        """取消订阅深度摆盘ARCABOOK推送消息"""
        response = self._open_api.hq_unsubscribe(topic_id=HQSubscribeTopicId.ARCA_BOOK,
                                                 security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"取消订阅深度摆盘ARCABOOK推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_unsubscribe_arca_book, response: {response}")

    def hq_unsubscribe_total_view_book(self):
        """取消订阅深度摆盘TOTALVIEW推送消息"""
        response = self._open_api.hq_unsubscribe(topic_id=HQSubscribeTopicId.TOTALVIEW_BOOK,
                                                 security_list=[{"dataType": DataType.HK_STOCK, "code": "00700.HK"}])
        if response is not None and response.get('ok'):
            print(f"取消订阅深度摆盘TOTALVIEW推送消息：{response.get('data')}")
        else:
            print(f"Fail to hq_unsubscribe_total_view_book, response: {response}")

    def notify_callback(self, pb_notify):
        """
        定义处理消息推送的callback
        :param pb_notify  参考 PBNotify.proto
        TradeStockDeliverMsgType = 1
        FuturesTradeStockDeliverMsgType = 2
        BasicQotNotifyMsgType = 20003
        BrokerQueueNotifyMsgType = 20002
        OrderBookNotifyMsgType = 20001
        TickerNotifyMsgType = 20004
        """
        notify_msg_type = pb_notify.notifyMsgType
        payload = parse_payload(pb_notify)
        if notify_msg_type == TradeStockDeliverMsgType or notify_msg_type == FuturesTradeStockDeliverMsgType:
            print(f"交易回报推送：股票代码: {payload.stockCode}, 订单号: {payload.recordNo}, entrustStatus: {payload.entrustStatus}")
        elif notify_msg_type == BasicQotNotifyMsgType:
            print(f"行情基础报价推送：股票代码: {payload.security.code}, 最新价: {payload.basicQot.lastPrice}, 成交量: {payload.basicQot.volume}")
        elif notify_msg_type == BrokerQueueNotifyMsgType:
            level = 0
            name = ""
            if payload.brokerList:
                level = payload.brokerList[0].level
                name = payload.brokerList[0].name
            print(f"行情买卖经纪推送：股票代码: {payload.security.code}, 档位: {level}, 经纪商名称: {name}")
        elif notify_msg_type == OrderBookNotifyMsgType:
            level = 0
            price = ""
            if payload.orderBookList:
                level = payload.orderBookList[0].level
                price = payload.orderBookList[0].price
            print(f"行情买卖档推送：股票代码: {payload.security.code}, 档位: {level}, 价格: {price}")
        elif notify_msg_type == TickerNotifyMsgType:
            mkt_tm_str = ''
            if payload.ticker.mktTmType == MktTmType.MID_SESSION:
                mkt_tm_str = "(盘中)"
            elif payload.ticker.mktTmType == MktTmType.PRE_MARKET:
                mkt_tm_str = "(盘前)"
            elif payload.ticker.mktTmType == MktTmType.AFTER_HOURS:
                mkt_tm_str = "(盘后)"
            print(f"行情逐笔推送{mkt_tm_str}：股票代码: {payload.security.code}, 价格: {payload.ticker.price}, 时间戳: {payload.ticker.timestamp}")
        else:
            print(f"未知消息推送类型：msgType: {notify_msg_type}, payload: {payload}")

    def stop(self):
        """退出API上下文环境"""
        self._open_api.stop()


if __name__ == '__main__':
    # 1、配置启动参数
    params = {
        "ip": "127.0.0.1",  # 请求IP（* 开发者部署 HS Gateway 所在服务器IP *）
        "http_port": 11111,  # 业务请求端口（* 开发者部署 HS Gateway 设置的HTTP端口 *）
        "tcp_port": 11112,  # 消息推送端口（* 开发者部署 HS Gateway 设置的TCP端口 *）
        "logging_filename": None  # 日志文件路径（值为None则打印日志到console） e.g. "/tmp/hs.log"
    }


    def help():
        print("====================================================")
        print("HTTP 请求地址   " + "http://" + params["ip"] + ":" + str(params["http_port"]))
        print("TCP 长连接地址  " + params["ip"] + ":" + str(params["tcp_port"]))
        print("OpenAPI在线文档 https://quant-open.hstong.com/api-docs/")
        print("Q,q  - 终止")
        print("help - 帮助（查看指令）")
        print("接口名: query_rate_list                      接口描述: 配置接口-查询汇率列表")
        print("接口名: trade_login                          接口描述: 交易接口-交易登录")
        print("接口名: trade_logout                         接口描述: 交易接口-交易登出")
        print("接口名: query_margin_fund_info               接口描述: 交易接口-查询客户资金信息")
        print("接口名: query_holds_list                     接口描述: 交易接口-查询持仓股票列表")
        print("接口名: query_buy_amount                     接口描述: 交易接口-查询最大可买")
        print("接口名: query_sell_amount                    接口描述: 交易接口-查询最大可卖")
        print("接口名: query_real_fund_jour_list            接口描述: 交易接口-查询当日资金流水")
        print("接口名: query_history_fund_jour_list         接口描述: 交易接口-查询历史资金流水")
        print("接口名: entrust                              接口描述: 交易接口-下单")
        print("接口名: cancel_entrust                       接口描述: 交易接口-撤单")
        print("接口名: batch_cancel_entrust                 接口描述: 交易接口-批量撤单")
        print("接口名: change_entrust                       接口描述: 交易接口-改单")
        print("接口名: query_max_available_asset            接口描述: 交易接口-查询最大可用资产")
        print("接口名: query_real_entrust_list              接口描述: 交易接口-查询当日委托")
        print("接口名: query_history_entrust_list           接口描述: 交易接口-查询历史委托")
        print("接口名: query_real_deliver_list              接口描述: 交易接口-查询当日成交")
        print("接口名: query_history_deliver_list           接口描述: 交易接口-查询历史成交")
        print("接口名: query_real_cond_order_list           接口描述: 交易接口-查询当日条件单")
        print("接口名: query_history_cond_order_list        接口描述: 交易接口-查询历史条件单")
        print("接口名: query_before_and_after_support       接口描述: 交易接口-查询是否支持盘前盘后交易")
        print("接口名: query_stock_short_info               接口描述: 交易接口-查询股票沽空信息")
        print("接口名: query_margin_full_info               接口描述: 交易接口-查询股票融资融券数据")
        print("接口名: futures_query_fund_info              接口描述: 交易接口-期货-查询客户资金信息")
        print("接口名: futures_query_holds_list             接口描述: 交易接口-期货-查询持仓列表")
        print("接口名: futures_query_max_buy_sell_amount    接口描述: 交易接口-期货-查询最大可买可卖")
        print("接口名: futures_query_product_info           接口描述: 交易接口-期货-查询合约产品信息")
        print("接口名: futures_entrust                      接口描述: 交易接口-期货-下单")
        print("接口名: futures_cancel_entrust               接口描述: 交易接口-期货-撤单")
        print("接口名: futures_change_entrust               接口描述: 交易接口-期货-改单")
        print("接口名: futures_query_real_entrust_list      接口描述: 交易接口-期货-查询今日委托")
        print("接口名: futures_query_history_entrust_list   接口描述: 交易接口-期货-查询历史委托")
        print("接口名: futures_query_real_deliver_list      接口描述: 交易接口-期货-查询今日成交")
        print("接口名: futures_query_history_deliver_list   接口描述: 交易接口-期货-查询历史成交")
        print("接口名: trade_subscribe                      接口描述: 交易接口-订阅交易推送消息")
        print("接口名: trade_unsubscribe                    接口描述: 交易接口-取消订阅交易推送消息")
        print("接口名: query_hq_basic_qot                   接口描述: 行情接口-基础报价")
        print("接口名: query_hq_broker                      接口描述: 行情接口-买卖经纪摆盘")
        print("接口名: query_hq_order_book                  接口描述: 行情接口-查询买卖档")
        print("接口名: query_hq_ticker                      接口描述: 行情接口-查询最近多少条的逐笔列表")
        print("接口名: query_hq_kline                       接口描述: 行情接口-K线数据")
        print("接口名: query_hq_time_share                  接口描述: 行情接口-查询分时数据")
        print("接口名: query_hq_us_option_chain_expire_date 接口描述: 行情接口-查询期权过期日列表")
        print("接口名: query_hq_us_option_chain_code        接口描述: 行情接口-查询期权Code列表")
        print("接口名: hq_subscribe_basic_qot               接口描述: 行情接口-订阅基础行情推送消息")
        print("接口名: hq_subscribe_ticker                  接口描述: 行情接口-订阅逐笔推送消息")
        print("接口名: hq_subscribe_broker                  接口描述: 行情接口-订阅买卖经纪推送消息")
        print("接口名: hq_subscribe_order_book              接口描述: 行情接口-订阅买卖档推送消息")
        print("接口名: hq_subscribe_total_view_book         接口描述: 行情接口-订阅深度摆盘TOTALVIEW推送消息")
        print("接口名: hq_subscribe_arca_book               接口描述: 行情接口-订阅深度摆盘ARCABOOK推送消息")
        print("接口名: hq_unsubscribe_basic_qot             接口描述: 行情接口-取消订阅基础行情推送消息")
        print("接口名: hq_unsubscribe_ticker                接口描述: 行情接口-取消订阅逐笔推送消息")
        print("接口名: hq_unsubscribe_broker                接口描述: 行情接口-取消订阅买卖经纪推送消息")
        print("接口名: hq_unsubscribe_order_book            接口描述: 行情接口-取消订阅买卖档推送消息")
        print("接口名: hq_unsubscribe_total_view_book       接口描述: 行情接口-取消订阅深度摆盘TOTALVIEW推送消息")
        print("接口名: hq_unsubscribe_arca_book             接口描述: 行情接口-取消订阅深度摆盘ARCABOOK推送消息")
        print(
            "接口名: stop                                 接口描述: ！！！程序退出，该函数将退出交易登录，并断开TCP链接！！！")
        print("====================================================")


    help()

    # 2、初始化交易API对象、增加消息推送回调函数
    open_api_demo = OpenAPIDemo(**params).add_notify_callback()
    # 3、启动交易API上下文，并会初始化连接、交易登录
    open_api_demo.start()

    # 4、检查连接状态
    if open_api_demo.check_alive():
        # 5、命令形式展示
        while True:
            method_name = input("请输入需要查看的接口名:\n")
            f = getattr(open_api_demo, method_name, None)

            if method_name == "stop" or method_name == "Q" or method_name == "q":
                # 【！！！注意：调用该函数将退出登录，并断开TCP链接。请在停止程序时调用！！！】
                open_api_demo.stop()
                exit()
            if method_name == "help":
                # 打印help信息
                help()
            elif f != None and callable(f):
                # 调用demo方法
                parameter_count = len(f.__code__.co_varnames)
                print("parameter_count=", parameter_count)
                f()
            else:
                print("接口名输入有误，请参考help重新输入！")
    else:
        # 【！！！注意：调用该函数将退出登录，并断开TCP链接。请在停止程序时调用！！！】
        open_api_demo.stop()
        exit()
